#!/usr/bin/env python

import os
import sys
#from rules import RULES

sys.path.append("classes")

import colony

MAX_TECHNOLOGIES = 0xCB             # 203 technologies

def get_int(b0, b1, b2 = 0, b3 = 0):
    return b0 + (b1 << 8) + (b2 << 16) + (b3 << 24)
# end func get_int

def read_short_int(data, offset):
    return ord(data[offset]) + (ord(data[offset+1]) << 8)
# end func read_short_int

def read_long_int(data, offset):
    return ord(data[offset]) + (ord(data[offset+1]) << 8) + (ord(data[offset+2]) << 16) + (ord(data[offset+3]) << 24)
# end func read_short_int

def read_byte(data, offset):
    return ord(data[offset])
# end func read_signed_byte

def read_signed_byte(data, offset):
    v = ord(data[offset])
    if v > 0x7f:
	v -= 0x100
    return v
# end func read_signed_byte

def read_string(data, offset, length):
    return data[offset:offset + length].rstrip(chr(0) + chr(1) + chr(2) + chr(3)).split(chr(0))[0]
# end func read_string


"""
"""
def list_savegames():
    from stat import ST_MTIME
    list = []
    for n in range(1,11):
	filename = "SAVE%i.GAM" % n
#	filesize = os.path.getsize(filename)
	savefile = open(filename, 'rb')
	data = savefile.read(64)
	savefile.close()
	info = read_game(data)
	sg_stat = os.stat(filename)
	info['ST_MTIME'] = sg_stat[ST_MTIME]
#	info['fstat'] = os.fstat(filename)
#	info['lstat'] = os.lstat(filename)
	list.append(info)
    return list
# end func list_savegames
"""
    read_game
"""
def read_game(data):
    return {
	'name':					read_string(data, 0x04, 23).lstrip(chr(0) + chr(1) + chr(2) + chr(3)),
	'stardate':				read_short_int(data, 0x29),
	'end_of_turn_summary':			read_byte(data, 0x2E),
	'end_of_turn_wait':			read_byte(data, 0x2F),
	'random_events':			read_byte(data, 0x30),
	'enemy_moves':				read_byte(data, 0x31),
	'expanding_help':			read_byte(data, 0x32),
	'auto_select_ships':			read_byte(data, 0x33),
	'animations':				read_byte(data, 0x34),
	'auto_select_colony':			read_byte(data, 0x35),
	'show_relocation_lines':		read_byte(data, 0x36),
	'show_gnn_report':			read_byte(data, 0x37),
	'auto_delete_trade_good_housing':	read_byte(data, 0x38),
	'auto_save_game':			read_byte(data, 0x39),
	'show_only_serious_turn_summary':	read_byte(data, 0x3A),
	'ship_initiative':			read_byte(data, 0x3B)
    }
# end func read_game

def read_galaxy(data):
    return {
	'size_factor':	read_byte(data, 0x31be4),
	'width':	get_int(ord(data[0x31be9]), ord(data[0x31bea])),
	'height':	get_int(ord(data[0x31beb]), ord(data[0x31bec]))
    }

def read_heroes(data, debug = False):
    HEROES_DATA_OFFSET	= 0x19a9b
    HERO_RECORD_SIZE	= 0x3b 		# = 59
    heroes = []
    for i in range(67):
        offset = HEROES_DATA_OFFSET + (HERO_RECORD_SIZE * i)

	if debug:
	    print "hero @ %x" % offset

	heroes.append({
            'id':               i,
	    'name':		data[offset:offset + 0x0f].rstrip(chr(0)).split(chr(0))[0],
    	    'title':		data[offset + 0x0f:offset + 0x23].rstrip(chr(0)).split(chr(0))[0],
            'type':		read_byte(data, offset + 0x23),
            'experience':	read_short_int(data, offset + 0x24),
            'common_skills':    read_long_int(data, offset + 0x26),
            'special_skills':   read_long_int(data, offset + 0x2a),
            'tech1':            read_byte(data, offset + 0x2e),
            'tech2':            read_byte(data, offset + 0x2f),
            'tech3':            read_byte(data, offset + 0x30),
            'picture':          read_byte(data, offset + 0x31),
            'skill_value':	read_short_int(data, offset + 0x32),
	    'level':		read_byte(data, offset + 0x34),
	    'location':		read_short_int(data, offset + 0x35),
	    'eta':		read_byte(data, offset + 0x37),
	    'level_up':		read_byte(data, offset + 0x38),
	    'status':		read_byte(data, offset + 0x39),
	    'player':		read_byte(data, offset + 0x3a)
	})

    if debug:
        i += 1 
        offset = 0x19a9b + (59 * i)
        print "STOP hero @ %x" % offset
    return heroes
# end func read_heroes

def read_colony_population(data, colony_offset):
    population	= ord(data[colony_offset + 0x0A])
#    colonists_raw = []
    colonists = {0x02: [], 0x03: [], 0x82: []}
    for i in range(population):
	offset = colony_offset + 0x0C + (4 * i)
	t = (read_byte(data, offset) & 0x80) + (read_byte(data, offset + 1) & 3)
        colonists[t].append({
	    'a':	read_byte(data, offset),
	    'b':	read_byte(data, offset + 1),
	    'c':	read_byte(data, offset + 2),
	    'd':	read_byte(data, offset + 3),
#	    'type':	(ord(data[offset]) & 0x80) + (ord(data[offset + 1]) & 3),
	    'r1':	(read_byte(data, offset) & 0x70) >> 4,
	    'race':	(read_byte(data, offset) & 0x07)
	})
    return colonists
# end func read_colony_population

def read_colonies(data, planets, debug = False):
    COLONIES_DATA_OFFSET	= 0x0025d
    COLONY_RECORD_SIZE		= 361
    c = get_int(ord(data[0x0000025b]), ord(data[0x0000025c]))

    if debug:
	print "Number of colonies to read: %i" % c

    colonies = []

    for i in range(c):
	offset	= COLONIES_DATA_OFFSET + (COLONY_RECORD_SIZE * i)
	colony = Colony()
	colony.import_from_moo2(i, data[offset:offset + COLONY_RECORD_SIZE])
#	owner	= ord(data[offset])
	colonies.append(colony)
    return colonies
# end func read_colonies
	
def read_players(data, debug = False):
    #
    # Players
    #

    players = []

    if debug:
	print
	print "=== Players ==="

    for i in range(8):
        offset		= 0x01aa0f + (3753 * i)
	if debug:
	    print "player @ %x" % offset

        players.append({
	    'emperor':		data[offset:offset + 0x0F].rstrip(chr(0)).split(chr(0))[0],
	    'race':		data[offset + 0x14:offset + 0x24].rstrip(chr(0)).split(chr(0))[0],
#	    'race':		ord(data[offset + 0x24]),
	    'picture':		read_byte(data, offset + 0x24),
	    'color':		read_byte(data, offset + 0x25),
	    'personality':	ord(data[offset + 0x26]),
	    'objective':	ord(data[offset + 0x27]),			# 100 = Human Player
	    # 0x28 ~ 0x2f	unknown
	    'tax_rate':		ord(data[offset + 0x30]),
	    'BC':		get_int(ord(data[offset + 0x31]), ord(data[offset + 0x32]), ord(data[offset + 0x33]), ord(data[offset + 0x34])),
	    'total_frighters':	get_int(ord(data[offset + 0x35]), ord(data[offset + 0x36])),
	    'used_frighters':	get_int(ord(data[offset + 0x37]), ord(data[offset + 0x38])),
	    'command_points':	get_int(ord(data[offset + 0x39]), ord(data[offset + 0x3A])),
	    # 0x3B ~ 0xA8	unknown
	    # 0xA0		?
	    # 0xA1		?
	    # 0xA2		?
	    # 0xA3		?
	    # 0xA4		?
	    # 0xA5		?
	    # 0xA6		?
	    # 0xA7		?
	    # 0xA8		?
	    'total_production':	get_int(ord(data[offset + 0xA9]), ord(data[offset + 0xAA])),
	    'RP':		get_int(ord(data[offset + 0xAB]), ord(data[offset + 0xAC])),
	    # 0xAD		unknown	...
#		... pracant na vedce ... 0x10 -> 0x0f
	    # 0xAE		unknown
	    'food':		get_int(ord(data[offset + 0x0AF]), ord(data[offset + 0x0B0])),
            'yearly_bc':	get_int(ord(data[offset + 0x0B1]), ord(data[offset + 0x0B2])),
            'research_progress':get_int(ord(data[offset + 0x1EA]), ord(data[offset + 0x1EB])),
            'research_area':    ord(data[offset + 0x320]),
#               0x00 = None
#		0x04 = Construction
#		0x07 = Force Fields
#		0x0a = Sociology
#		0x12 = Biology
#		0x16 = Chemistry
#		0x1c = Computers
#		0x37 = Power
#		0x39 = Physics
            'research_item':    	ord(data[offset + 0x321]),
	    'racepicks':	{
	    			    'goverment':		ord(data[offset + 0x89E]),
				    'population':		read_signed_byte(data, offset + 0x89F),
				    'farming':			read_signed_byte(data, offset + 0x8A0),
				    'industry':			read_signed_byte(data, offset + 0x8A1),
				    'science':			read_signed_byte(data, offset + 0x8A2),
				    'money':			read_signed_byte(data, offset + 0x8A3),
				    'ship_defense':		read_signed_byte(data, offset + 0x8A4),
				    'ship_attack':		read_signed_byte(data, offset + 0x8A5),
				    'ground_combat':		read_signed_byte(data, offset + 0x8A6),
				    'spying':			read_signed_byte(data, offset + 0x8A7),
				    'low_g':			ord(data[offset + 0x8A8]),
				    'high_g':			ord(data[offset + 0x8A9]),
				    'aquatic':			ord(data[offset + 0x8AA]),
				    'subterranean':		ord(data[offset + 0x8AB]),
				    'large_home_world':		ord(data[offset + 0x8AC]),
				    'rich_home_world':		ord(data[offset + 0x8AD]),
				    # where is poor_home_worlds?
				    'artifacts_home_world':	ord(data[offset + 0x8AE]),
				    'cybernetic':		ord(data[offset + 0x8AF]),
				    'lithovore':		ord(data[offset + 0x8B0]),
				    'repulsive':		ord(data[offset + 0x8B1]),
				    'charismatic':		ord(data[offset + 0x8B2]),
				    'uncreative':		ord(data[offset + 0x8B3]),
				    'creative':			ord(data[offset + 0x8B4]),
				    'tolerant':			ord(data[offset + 0x8B5]),
				    'fantastic_traders':	ord(data[offset + 0x8B6]),
				    'telepathic':		ord(data[offset + 0x8B7]),
				    'lucky':			ord(data[offset + 0x8B8]),
				    'omniscience':		ord(data[offset + 0x8B9]),
				    'stealthy_ships':		ord(data[offset + 0x8BA]),
				    'trans_dimensional':	ord(data[offset + 0x8BB]),
				    'warlord':			ord(data[offset + 0x8BC])
				},
#	    'technologies':	[],
	    'known_techs':	[],
	    'prototypes':	[],
	    'tributes':		[]
	})

        for ii in range(MAX_TECHNOLOGIES):
            offset2 = offset + 0x117 + ii
#            players[-1]['technologies'].append(ord(data[offset2]))
            if ord(data[offset2]) == 3:
                players[-1]['known_techs'].append(ii + 1)

#	0x1e2
#
        for ii in range(6):
            offset2 = offset + 0x325 + (ii * 0x63)
#            proto_name = data[offset2:offset2 + 0x0F].rstrip(chr(0)).split(chr(0))[0]
#            players[-1]['prototypes'].append({
#		'name':	data[offset2:offset2 + 0x0F].rstrip(chr(0)).split(chr(0))[0]
#	    })

            proto = read_ship_design(data, offset2)
#            if proto['name']:
#                print("")
#                print("=== Prototype @%i: ===" % offset2)
#                print(proto)
#                print("=== /Prototype ===")
#                print("")

            players[-1]['prototypes'].append(proto)

	for ii in range(7):
	    offset2 = offset + 0x649 + (ii)
	    players[-1]['tributes'].append(ord(data[offset2]))

    if debug:
	i += 1 
        offset = 0x01aa0f + (3753 * i)
	print "STOP player @ %x" % offset

    players.append({
	'race':		"Antareans",
	'emperor':	"?"
    })
    players.append({
	'race':		"Guardian",
	'emperor':	"?"
    })
    players.append({
	'race':		"Amoeba",
	'emperor':	"?"
    })
    players.append({
	'race':		"Crystal",
	'emperor':	"?"
    })
    players.append({
	'race':		"Dragon",
	'emperor':	"?"
    })
    players.append({
	'race':		"Eel",
	'emperor':	"?"
    })
    players.append({
	'race':		"Hydra",
	'emperor':	"?"
    })

    return players
#end func read_players

def read_stars(data, debug = False):
    #       http://www.spheriumnorth.com/orion-forum/nfphpbb/viewtopic.php?p=149
    SOLAR_SYSTEMS_COUNT_OFFSET	= 0x17ad1
    SOLAR_SYSTEMS_DATA_OFFSET	= 0x17ad3
    SOLAR_SYSTEM_RECORD_SIZE	= 0x71
    systems = []
    c = ord(data[SOLAR_SYSTEMS_COUNT_OFFSET])
    for i in range(c):
#    for i in range(72):
        offset = SOLAR_SYSTEMS_DATA_OFFSET + (SOLAR_SYSTEM_RECORD_SIZE * i)
	if debug:
	    print "system @ %x" % offset
        systems.append({
	    'id':			i,
            'name':     		read_string(data, offset, 15),
            'x':        		read_short_int(data, offset + 0x0f),
            'y':        		read_short_int(data, offset + 0x11),
            'size':     		read_byte(data, offset + 0x13),
	    'owner':			read_byte(data, offset + 0x14),		# primary owner
	    'pict_type':		read_byte(data, offset + 0x15),
            'class':    		read_byte(data, offset + 0x16),
	    'last_planet_selected':	[
					    read_byte(data, offset + 0x17),
					    read_byte(data, offset + 0x18),
	    				    read_byte(data, offset + 0x19),
					    read_byte(data, offset + 0x1A),
	    				    read_byte(data, offset + 0x1B),
					    read_byte(data, offset + 0x1C),
					    read_byte(data, offset + 0x1D),
					    read_byte(data, offset + 0x1E)
					],
	    'black_hole_blocks':	[
					],
	    '0x1f':		ord(data[offset + 0x1f]),
	    '0x20':		ord(data[offset + 0x20]),
	    '0x21':		ord(data[offset + 0x21]),
	    '0x22':		ord(data[offset + 0x22]),
	    '0x23':		ord(data[offset + 0x23]),
	    '0x24':		ord(data[offset + 0x24]),
	    '0x25':		ord(data[offset + 0x25]),
	    '0x26':		ord(data[offset + 0x26]),
	    '0x27':		ord(data[offset + 0x27]),
            'special':  	ord(data[offset + 0x28]),
            'wormhole': 	ord(data[offset + 0x29]),
	    '0x2a':		ord(data[offset + 0x2a]),	# blockaded? ( 0 | 1 )
	    '0x2b':		ord(data[offset + 0x2b]),
	    '0x2c':		ord(data[offset + 0x2c]),
	    '0x2d':		ord(data[offset + 0x2d]),
	    '0x2e':		ord(data[offset + 0x2e]),
	    '0x2f':		ord(data[offset + 0x2f]),
	    '0x30':		ord(data[offset + 0x30]),
	    '0x31':		ord(data[offset + 0x31]),
	    '0x32':		ord(data[offset + 0x32]),
	    'visited':		ord(data[offset + 0x33]),	# bitmask as boleans for each player
	    '0x34':		ord(data[offset + 0x34]),
	    '0x35':		ord(data[offset + 0x35]),
	    '0x36':		ord(data[offset + 0x36]),
	    '0x37':		ord(data[offset + 0x37]),
	    '0x38':		ord(data[offset + 0x38]),
            'indictor': 	ord(data[offset + 0x39]),	# 0 = none, 1-8 = owner player 0-7  ( http://code.google.com/p/moo2x/wiki/star )
	    '0x3a':		ord(data[offset + 0x3a]),
	    '0x3b':		ord(data[offset + 0x3b]),
	    '0x3c':		ord(data[offset + 0x3c]),
	    '0x3d':		ord(data[offset + 0x3d]),
	    '0x3e':		ord(data[offset + 0x3e]),
            'artemis':  	ord(data[offset + 0x3f]),	# 0 = none, 1-8=owner player 0-7 ( http://code.google.com/p/moo2x/wiki/star )
	    '0x40':		ord(data[offset + 0x40]),	# Star has dimensional portal ?
	    '0x41':		ord(data[offset + 0x41]),
	    '0x42':		ord(data[offset + 0x42]),
	    '0x43':		ord(data[offset + 0x43]),
	    '0x44':		ord(data[offset + 0x44]),
	    '0x45':		ord(data[offset + 0x45]),
	    '0x46':		ord(data[offset + 0x46]),
	    '0x47':		ord(data[offset + 0x47]),
	    '0x48':		ord(data[offset + 0x48]),
	    '0x49':		ord(data[offset + 0x49]),
            'objects':   	[
				    get_int(ord(data[offset + 0x4a]), ord(data[offset + 0x4b])),
        			    get_int(ord(data[offset + 0x4c]), ord(data[offset + 0x4d])),
        			    get_int(ord(data[offset + 0x4e]), ord(data[offset + 0x4f])),
        			    get_int(ord(data[offset + 0x50]), ord(data[offset + 0x51])),
        			    get_int(ord(data[offset + 0x52]), ord(data[offset + 0x53]))
				],
#		??? Relocation star id (0-7 player #, not sure about size of array. ) 
	    '0x54':		ord(data[offset + 0x54]),
	    '0x55':		ord(data[offset + 0x55]),
	    '0x56':		ord(data[offset + 0x56]),
	    '0x57':		ord(data[offset + 0x57]),
	    '0x58':		ord(data[offset + 0x58]),
	    '0x59':		ord(data[offset + 0x59]),
	    '0x5a':		ord(data[offset + 0x5a]),
	    '0x5b':		ord(data[offset + 0x5b]),
	    '0x5c':		ord(data[offset + 0x5c]),
	    '0x5d':		ord(data[offset + 0x5d]),
	    '0x5e':		ord(data[offset + 0x5e]),
	    '0x5f':		ord(data[offset + 0x5f]),
	    '0x60':		ord(data[offset + 0x60]),
	    '0x61':		ord(data[offset + 0x61]),
	    '0x62':		ord(data[offset + 0x62]),
	    '0x63':		ord(data[offset + 0x63]),
#		unknown:
	    '0x64':		ord(data[offset + 0x64]),
	    '0x65':		ord(data[offset + 0x65]),
	    '0x66':		ord(data[offset + 0x66]),
	    '0x67':		ord(data[offset + 0x67]),
	    '0x68':		ord(data[offset + 0x68]),
	    '0x69':		ord(data[offset + 0x69]),
	    '0x6a':		ord(data[offset + 0x6a]),
	    '0x6b':		ord(data[offset + 0x6b]),
	    '0x6c':		ord(data[offset + 0x6c]),
	    '0x6d':		ord(data[offset + 0x6d]),
	    '0x6e':		ord(data[offset + 0x6e]),
	    '0x6f':		ord(data[offset + 0x6f]),
	    '0x70':		ord(data[offset + 0x70])
            })
    if debug:
	i += 1
	offset = 0x17ad3 + (SOLAR_SYSTEM_RECORD_SIZE * i)
	print "STOP system @ %x" % offset
#    blah
    return systems
# end func read_stars

def read_planets(data, systems, debug = False):
#    PLANETS_COUNT_OFFSET	= 0x162e7
    PLANETS_DATA_OFFSET		= 0x162e9
    PLANET_RECORD_SIZE		= 0x11
    planets = []

    c = 0
    for system in systems:
	for o in system['objects']:
	    if o != 0xffff:
		c += 1

#    c = get_int(ord(data[PLANETS_COUNT_OFFSET]), ord(data[PLANETS_COUNT_OFFSET + 1]))
    c = 250

    if debug:
	print "Reading planetary objects ..."
        print "    count: %i" % c

    for i in range(c):
        offset = PLANETS_DATA_OFFSET + (PLANET_RECORD_SIZE * i)
#        system_number = ord(data[offset + 0x02])
#        position = ord(data[offset + 0x03])
        planets.append({
            'colony':           read_short_int(data, offset),	# 0xffff = no colony here
	    'parent_star':	read_byte(data, offset + 0x02),
	    'position':		read_byte(data, offset + 0x03),
            'type':             read_byte(data, offset + 0x04),
            'size':             read_byte(data, offset + 0x05),
            'gravity':          read_byte(data, offset + 0x06),
            'group':            read_byte(data, offset + 0x07),   # not used
            'terrain':          read_byte(data, offset + 0x08),
            'picture':          read_byte(data, offset + 0x09),   # Background image on colony screen (0-5=image in planets.lbx)
            'minerals':         read_byte(data, offset + 0x0a),
            'foodbase':         read_byte(data, offset + 0x0b),
            'terraformations':  read_byte(data, offset + 0x0c),
            'max_farms':        read_byte(data, offset + 0x0d),   # unknown (Initial value is based on Planet Size but changes if colonized), 2=tiny, 4=small, 5=med, 7=large, A=huge
            'max_population':   read_byte(data, offset + 0x0e),
            'special':          read_byte(data, offset + 0x0f),
            'flags':            read_byte(data, offset + 0x10)    # (bit 2 = Soil Enrichment)
        })
    return planets
# end func read_planets

def read_ship_design(data, offset):
    MAX_WEAPONS = 8
    weapons = []
    for i in range(MAX_WEAPONS):
        offset2 = offset + 0x1C + (8 * i)
        weapon = read_byte(data, offset2)
        if (weapon > 0):
            weapons.append({
                'weapon':		weapon,
                'count':		read_byte(data, offset2 + 0x02),
                'current_count':	read_byte(data, offset2 + 0x03),
                'arc':			read_byte(data, offset2 + 0x04),
                'beam_mods':		read_byte(data, offset2 + 0x05),
                'missile_mods':		read_byte(data, offset2 + 0x06),
                'ammo':			read_byte(data, offset2 + 0x07)
            })
    return {
            'name':		read_string(data, offset, 16),
            'size':		read_byte(data, offset + 0x10),
    	    'type':		read_byte(data, offset + 0x11),
            'shield':		read_byte(data, offset + 0x12),
            'drive':		read_byte(data, offset + 0x13),
	    'speed':		read_byte(data, offset + 0x14),
            'computer':		read_byte(data, offset + 0x15),
            'armor':		read_byte(data, offset + 0x16),
	    'special_devices':	data[offset + 0x17:offset + 0x1C],	# char  special_device_flags[(MAX_SPECIALS+7)/8];
	    'weapons':		weapons,
            'picture':		read_byte(data, offset + 0x5C),
            'builder':		read_byte(data, offset + 0x5D),		# or previous owner?
            'cost':		read_short_int(data, offset + 0x5E),
	    'combat_speed':	read_byte(data, offset + 0x60),
	    'build_date':	read_short_int(data, offset + 0x61)
    }
# end func read_ship_design

def read_ships(data, debug = False):
    SAVE_SHIP_COUNT_OFFSET    = 0x21f56
    SAVE_SHIPS_OFFSET         = 0x21f58
    SAVE_SHIP_RECORD_SIZE     = 0x81        # = 129
#    SAVE_SHIP_WEAPONS_OFFSET  = 0x1c
#    SAVE_SHIP_SPECIALS_OFFSET = 0x17

    c = get_int(ord(data[SAVE_SHIP_COUNT_OFFSET]), ord(data[SAVE_SHIP_COUNT_OFFSET + 1]))
    ships = []
    
    for i in range(c):
        offset          = SAVE_SHIPS_OFFSET + (SAVE_SHIP_RECORD_SIZE * i)
        if debug:
    	    print "ship @ %x" % offset
            
        ships.append({
	    'design':			read_ship_design(data, offset),
            'owner':			read_byte(data, offset + 0x63),
            'status':			read_byte(data, offset + 0x64),           # 0 = orbitting, 2 = traveling, 5 = destroyed?
            'location':			read_byte(data, offset + 0x65),
            'location_x':		read_byte(data, offset + 0x66),
            'x':			read_short_int(data, offset + 0x67),
            'y':			read_short_int(data, offset + 0x69),
            'group_has_navigator':	read_byte(data, offset + 0x6B),		# does the moving group have navigator?
            'travelling_speed':		read_byte(data, offset + 0x6C),		# possibly less than ftl_type
            'turns_left':		read_byte(data, offset + 0x6D),		# until arrival
	    'shield_damage_percent':	read_byte(data, offset + 0x6E),
	    'drive_damage_percent':	read_byte(data, offset + 0x6F),
	    'computer_damage':		read_byte(data, offset + 0x70),
            'crew_quality':		read_byte(data, offset + 0x71),
            'crew_experience':		read_short_int(data, offset + 0x72),
            'officer_id':		read_short_int(data, offset + 0x74),
	    'special_device_damage':	data[offset + 0x76:offset + 0x7B],	# bit flag array
	    'armor_damage':		read_short_int(data, offset + 0x7B),
	    'structural_damage':	read_short_int(data, offset + 0x7D),
	    'mission':			read_byte(data, offset + 0x7F),		# AI field
	    'just_built':		read_byte(data, offset + 0x80),
        })

    if debug:
        i += 1		
        offset          = SAVE_SHIPS_OFFSET + (SAVE_SHIP_RECORD_SIZE * i)
        print "STOP ship @ %x" % offset
    return ships
# end func read_ships

def read_savegame(filename):
    pass
    print "Loading savegame '%s'" % filename
    
    filesize = os.path.getsize(filename)
    savefile = open(filename, 'rb')
    data = savefile.read(filesize)
    savefile.close()

    game	= read_game(data)
    galaxy	= read_galaxy(data)
    heroes	= read_heroes(data)			# complete
    players	= read_players(data)
    stars	= read_stars(data)
    planets	= read_planets(data, stars)		# complete
    colonies	= read_colonies(data, planets)
    ships	= read_ships(data)			# complete

#    for i in range(len(stars)):
    stars_by_coordinates = {}
#    for star in stars:

    for i in range(len(stars)):
	star = stars[i]
	k = "%i:%i" % (star['x'], star['y'])
	stars_by_coordinates[k] = star
	num = 0
	for object in star['objects']:
#	    print "object: %i" % object
	    if object != 0xffff:
		num += 1
		planets[object]['num'] = num

#    print "Attaching planets to colonies..."
    for i in range(len(colonies)):
	colony		= colonies[i]
	planet_id	= colony.planet_id
	if colony.owner() < 0xff:
#	    print "	... planet #%i to colony #%i" % (planet_id, i)
	    colony.assign_planet(planets[planet_id])
    	    colony.name = "%s %i" % (stars[planets[planet_id]['parent_star']]['name'], planets[planet_id]['num'])
#	else:
#	    print "	... colony #%i has owner 0xff" % (i)
#    print "	Finished"

    return {
	'game':			game,
	'galaxy':		galaxy,
	'heroes':		heroes,
	'players':		players,
	'stars':		stars,
	'stars_by_coordinates':	stars_by_coordinates,
	'planets':		planets,
	'colonies':		colonies,
	'ships':		ships
    }


if __name__ == "__main__":
    read_savegame("../moo2/SAVE8.GAM")



import pygame

import get_input

#import formulas

from _game_constants import *

def print_summary_item(data, key, title):
    if data.has_key(key) and data[key]:
	print "+     % 6s ... % 25s    +" % (str(data[key]), title)

TECH_REVIEW = None

##
#       DRAW
##
#def draw(GAME, terrain, picture, colony_name):
def draw(GAME, tech_review):
    from game import stardate

    DATA        = GAME['DATA']
#    RULES       = DATA['rules']
    ME          = DATA['me']


    DISPLAY     = GAME['DISPLAY']
    IMAGES      = GAME['IMAGES']
    FONTS	= GAME['FONTS']

    DICTIONARY	= GAME['DICTIONARY']
    
#    PLAYERS	= GAME['DATA']['players']

    DISPLAY.fill((0, 0, 0))
#    DISPLAY.fill((0x08, 0x38, 0x08))
    DISPLAY.blit(IMAGES['INFO_SCREEN']['panel'], (0, 0))

    DISPLAY.blit(FONTS['font_16_bold'].render(stardate(GAME['DATA']['galaxy']['stardate']), 1, (0x0C, 0x94, 0x0C)), (121, 24))

    DISPLAY.blit(IMAGES['INFO_SCREEN']['history_graph_off'], (21, 50))
    DISPLAY.blit(IMAGES['INFO_SCREEN']['tech_review_off'], (21, 77))
    DISPLAY.blit(IMAGES['INFO_SCREEN']['race_statistics_off'], (21, 102))
    DISPLAY.blit(IMAGES['INFO_SCREEN']['turn_summary_off'], (21, 128))
    DISPLAY.blit(IMAGES['INFO_SCREEN']['reference_off'], (21, 154))
#    DISPLAY.blit(IMAGES['INFO_SCREEN']['return'], (535, 434))

    DISPLAY.blit(IMAGES['APP_PICS'][0], (433, 115))
    DISPLAY.blit(IMAGES['APP_PICS'][155], (433, 115))

    buttons = []

    tech_carrets = []

    if tech_review == "achievements":

        # New Construction Types
        items = []
        for tech_id in [TECH_TRANSPORT, TECH_OUTPOST_SHIP, TECH_FREIGHTERS, TECH_COLONY_SHIP, TECH_COLONY_BASE]:
            if tech_id in ME.get_known_techs():
                items.append(tech_id)
        if len(items):
            tech_carrets.append({'title': "New Construction Types", 'items': items})

        # Spies
        items = []
        for tech_id in [TECH_TELEPATHIC_TRAINING, TECH_NEURAL_SCANNER, TECH_SPY_NETWORK]:
            if tech_id in ME.get_known_techs():
                items.append(tech_id)
        if len(items):
            tech_carrets.append({'title': "Spies", 'items': items})

    y = 63
    for carret in tech_carrets:
        if len(carret['items']):
            DISPLAY.blit(FONTS['font_12_bold'].render(carret['title'], 1, (0x0C, 0x94, 0x0C)), (223, y))
            y += 14
            for item in carret['items']:
                DISPLAY.blit(FONTS['font_12'].render(DICTIONARY['TECH_LIST'][item], 1, (0x0C, 0x94, 0x0C)), (233, y))
                y += 13
            y += 8


    pygame.display.flip()

    return buttons
# end func draw

##
#       RUN
##
def run(GAME):
    global TECH_REVIEW

    if TECH_REVIEW is None:
        TECH_REVIEW = "achievements"

    print
    print "### info_screen::run()"

    triggers = [
        {'action': "ESCAPE",    		'rect': pygame.Rect((547, 441), ( 64, 17))},
    ]

    triggers2 = draw(GAME, TECH_REVIEW)

    while True:
        event = get_input.get_input(triggers + triggers2)
        action = event['action']

        if (action == "ESCAPE"):
            return

        else:
            print "UNKNONW ACTION: " + action

# end func run



import socket
import pickle
import time


class GameSocket():

    def __init__(self, socket):
        self.__socket = socket
	self.set_buffer_size(4096)
#        self.set_buffer_size(65536)
#        self.set_buffer_size(131072)
#        self.set_buffer_size(262144)

    def set_buffer_size(self, buffer_size):
	self.__buffer_size = buffer_size

    def get_buffer_size(self):
	return self.__buffer_size

    def setsockopt(self, level, optname, value):
        self.__socket.setsockopt(level, optname, value)

    def setblocking(self, flag):
        self.__socket.setblocking(flag)

    def settimeout(self, value):
        self.__socket.settimeout(value)

    def gettimeout(self):
        return self.__socket.gettimeout()

    def bind(self, address):
        self.__socket.bind(address)

    def listen(self, backlog):
        self.__socket.listen(backlog)

    def accept(self):
        client_socket, (client_host, client_port) = self.__socket.accept()
  	client_socket.setblocking(1)
        client_socket.settimeout(1)
        return GameSocket(client_socket), (client_host, client_port)

    def connect(self, address):
        self.__socket.connect(address)

    def close(self):
        self.__socket.close()

    def recv(self):
        data = "";
        chunks = 0;
        while 1:
            try:
                s = self.__socket.recv(self.get_buffer_size())
                l = len(s)
#                print("GameSocket::recv() ... received %i bytes chunk" % l)
#                s = self.clientSocket.recv(SOCKET_BUFFER_SIZE)
            except socket.timeout:
#                print("Game_client::recv ... self.clientSocket.recv > timeout")
                s = None
            if not s:
                break
            data += s
            chunks += 1
#        print("Received from server: " + data)
        l = len(data)
#        print("GameSocket::recv() ... %i chunks received" % chunks)
#        print("GameSocket::recv() ... %i bytes received" % l)
        if not l:
#            print("! ERROR: GameSocket::recv() ... returning None?")
            return None
        else:
            return pickle.loads(data)
#        return data
    # /recv
#        data = self.__socket.recv(self.get_buffer_size())
#        if not data:
#            return None
#        else:
#            return pickle.loads(data)

    def send(self, data):
#	data = "DATA ... "  + pickle.dumps(data)
	data = pickle.dumps(data)
#        return self.__socket.sendall(data)
        size = len(data)
        sent = 0
	print("# GameSocket::send() ... sending %i bytes" % (size))
        while sent < size:
            chunk_size = self.__socket.send(data[sent:])
            sent = sent + chunk_size
            print("# GameSocket::send() ... %i bytes chunk sent (total %i / %i)" % (chunk_size, sent, size))
#        time.sleep(1)
        time.sleep(self.gettimeout())
        



#import os
import hashlib

import sys
import cli
import gui

import pygame
#from pygame.locals import *

#import lbx
import dictionary
import main_screen

import GameClient

import autoplayer

def show_usage(name, message):
    print
    print(message)
    print("Usage:")
    print(" %s -player <player id> [-h listen host] [-p listen port]" % name)
    print
# end func show_usage 


def main(argv):
    """
        MAIN
    """

    MOO2_DIR = "../moo2"

    default_options = {
        '-p':       	9999,
        '-h':       	"localhost",
        '-player':  	0
    }

    (OPTIONS, PARAMS) = cli.parse_cli_args(argv, default_options)

    argc = len(argv)

#    if argc < 2:
#        show_usage(argv[0], "ERROR: No option(s) given")
#        sys.exit(1)

    HOST	= OPTIONS['-h']
    PORT	= OPTIONS['-p']
    PLAYER_ID	= OPTIONS['-player']

    DEBUG = 0

    SOCKET_BUFFER_SIZE = 4096

    GUI = gui.Gui(pygame, "sans")
    GUI.load_lbx_archives()
    if not GUI.check_lbx_archives():
        print("")
        print("ERROR: some LBX files didn't match the expected MD5 checsum")
        print("")
        print("    The OpenMOO2 supports only original LBX files from MOO2 version 1.31")

    CLIENT = GameClient.GameClient(HOST, PORT, SOCKET_BUFFER_SIZE)
    CLIENT.connect()
    CLIENT.login(PLAYER_ID)

    server_status = CLIENT.get_server_status()
    print("# server_status = %s" % str(server_status))

    # automation for development
#    scenario = autoplayer.AutoPlayer(CLIENT)
#    scenario.play()

#    sys.exit(0)

#    WINDOW_WIDTH    = 640
#    WINDOW_HEIGHT   = 480
#    WINDOW_FLAGS	= 0
#    WINDOW_DEPTH	= 24
    WINDOW_CAPTION  = "OpenMOO2: PLAYER_ID = %s" % PLAYER_ID

    pygame.display.set_caption(WINDOW_CAPTION)

    DISPLAY = pygame.display.get_surface()

    ICON = pygame.image.load(MOO2_DIR + "/orion2-icon.png")
    pygame.display.set_icon(ICON)

    GUI.load_raw_palettes()
    GUI.init_palettes()

    PALETTES = GUI.get_palettes()
    FONTS = GUI.get_fonts()

    #    print "tested palette after loading: " + str(PALETTES['FONTS_02'])
    #    print

    IMAGES = GUI.get_images()

    GUI.load_splashscreen()
    GUI.draw_splashscreen(DISPLAY)
    pygame.display.flip()

    CLIENT.ping()

    GUI.load_graphic()

    #	COLONY VIEW

    GAME = {
	'GUI':			GUI,
        'DISPLAY':		DISPLAY,
        'PALETTES':		PALETTES,
        'IMAGES':		IMAGES,
        'FONTS':		FONTS,
        'DICTIONARY':           dictionary.get_dictionary(),
        'TURN':                 0,
        'close_game_menu':	False,
        'client':           	CLIENT
    }
    
    main_screen.run(GAME)
    GAME['client'].disconnect()
# end func main

if __name__ == "__main__":
    main(sys.argv)




__author__="peterman"
__date__ ="$Jan 10, 2009 11:23:18 PM$"

def get_dictionary():
    return {
    'COLONY_TYPES':             ["Colony", "Outpost"],
    'COLONY_ASSIGNMENT':        {0x00: "???", 0x01:"Agricultural Colony", 0x02: "Industrial Colony", 0x03: "Research Colony", 0xff: "Colony"},

    'RACE_PICTURES ':           ["Alkari", "Bulrathi", "Darlok", "Elerian", "Gnolam", "Human", "Klackon", "Meklar", "Mrrshan", "Psilon", "Sakkra", "Silicoid", "Trilarian"],

    'PLANET_TYPES':             ['?', 'Asteroids', 'Gas Giant', 'Planet', '??', '???', '????', '?????'],
    'PLANET_SIZES':             ['Tiny', 'Small', 'Medium', 'Large', 'Huge'],
    'PLANET_GRAVITIES':         ['Low G', 'Normal G', 'Heavy G'],
    'PLANET_TERRAINS':          ['Toxic', 'Radiated', 'Baren', 'Desert', 'Tundra', 'Ocean', 'Swamp', 'Arid', 'Terran', 'Gaia', 'k', 'l'],
    'PLANET_MINERALS':          ['Ultra Poor', 'Poor', 'Abundant', 'Rich', 'Ultra Rich'],
    'PLANET_SPECIALS':          ['-', 'Wormhole', 'Space Debris', 'Pirate Cache', 'Gold Deposits', 'Gem Deposits', 'Natives', 'Splinter', 'Hero', 'Monster', 'Artifacts', 'Orion'],

    'PLAYER_COLORS':		["red", "yellow", "green", "white", "blue", "brown", "purple", "orange"],
    'PLAYER_PERSONALITIES':	["Xenophobic", "Ruthless", "Aggressive", "Erratic", "Honorable", "Pacifist", "Dishonored"],
    'PLAYER_OBJECTIVES':	["Diplomat", "Militarist", "Expansionist", "Technologist", "Industrialist", "Ecologist"],
    'PLAYER_GOVERMENTS':	["Feudal", "Confederation", "Dictatorship", "Imperium", "Democracy", "Federation", "Unification", "Galactic Unification"],

    'SHIP_SIZES':               ["Frigate", "Destroyer", "Cruiser", "Battleship", "Titan", "Doomstar"],
    'SHIP_TYPES':               ["Combat Ship", "Colony Ship", "Transport Ship", "???", "Outpost Ship"],
    'DRIVES':                   ["-", "Nuclear", "Fusion", "Ion", "Antimatter", "Hyperdrive", "Interphased", "no unit"],
    'ARMORS':                   ["-", "Titanium", "Tritanium", "Zortrium", "Neutronium", "Adamantium", "Xentronium"],
    'SHIELDS':                  ["-", "class 1", "class 3", "class 5", "class 7", "class 10"],
    'COLONIST_TYPE':		{0x02: "farmer", 0x03: "scientist", 0x82: "worker"},
    'COMPUTERS':                ["-", "Electronic", "Optronic", "Positronic", "Cybertronic", "Moleculartronic"],
    'WEAPONS':                  ["None", "Mass Driver", "Gauss Cannon", "Laser Cannon", "Particle Beam", "Fusion Beam", "Ion Pulse Cannon", "Graviton Beam", "Neutron Blaster", "Phasor", "Disrupter", "Death Ray", "Plasma Cannon", "Spatial Compressor", "Nuclear Missile", "Merculite Missile", "Pulson Missile", "Zeon Missile", "Anti-Matter Torpedo", "Proton Torpedo", "Plasma Torpedo", "Nuclear Bomb", "Fusion Bomb", "Anti-Matter Bomb", "Neutronium Bomb", "Death Spore", "Bio Terminator", "Mauler Device", "Assault Shuttle", "Heavy Fighter", "Bomber", "Interceptor", "Stasis Field", "Anti-Missile Rocket", "Gyro Destabilizer", "Plasma Web", "Pulsar", "Black Hole Generator", "Stellar Converter", "Tractor Beam", "Dragon Breath", "Phasor Eye", "Crystal Ray", "Plasma Breath", "Plasma Flux", "Caustic Slime"],
    'WEAPON_ARCS':              ["-", "Forward", "Forward ext.", "", "Back ext.", "", "", "", "Back", "", "", "", "", "", "", "360", "x"],
    'WEAPON_MODS_BEAM':         ["-", "Heavy Mount", "Point Defense", "Armor piercing", "Continous", "No Range Dissipation", "Shield Piercing", "AutoFire"],
    'WEAPON_MODS_MISILLE':      ["-", "Enveloping", "Mirv", "ECCM", "Heavily Armored", "Fast", "Emimisions Guidance", "Overloaded"],

    'SHIP_EXP_LEVEL':           ["Green", "Regular", "Veteran", "Elite", "Ultra Elite"],

    'SHIP_SPECIALS':            [
			            ["unknown special 1", "Achilles Targeting Unit", "Augmented Engines", "Automated Repair Unit", "Battle Pods", "Battle Scanner", "Cloaking Device", "Damper Field"],
			            ["Displacement Device", "ECM Jammer", "Energy Absorber", "Extended Fuel Tanks", "Fast Missile Racks", "Hard Shields", "Heavy Armor", "High Energy Focus"],
			            ["Hyper X Capacitors", "Inertial Nullifier", "Inertial Stabilizer", "Lightning Field", "Multi-Phased Shields", "Multi-Wave ECM Jammer", "Phase Shifter", "Phasing Cloak"],
			            ["Quantum Dentonator", "Range Master Unit", "Reflection Field", "Reinforced Hull", "Scout Lab", "Security Stations", "Shield Capacitor", "Stealth Field"],
			            ["Structual Analyzer", "Sub Space Teleporter", "Time Warp Facilitator", "Transporters", "Troop Pods", "Warp Dissipator", "Wide Area Jammer", "unknown special 2"]
                                ],


    'STAR_SIZES':               ['Small', 'Medium', 'Large'],
    'STAR_CLASSES':             ['Blue', 'White', 'Yellow', 'Orange', 'Red', 'Gray', 'Black Hole'],

    'SYSTEM_SPECIALS':		["-", "stable wormhole", "space debris", "pirate cache", "gold deposits", "gem deposits", "natives", "splinter colony", "lost hero", "space monster", "ancient artifacts", "orion", "MAX_SYSTEM_AND_PLANET_SPECIALS"],

    'TECH_LIST':                ["Achilles Targeting Unit", "Adamantium Armor", "Advanced City Planning", "Advanced Damage Control", "Alien Management Center", "Android Farmers", "Android Scientists", "Android Workers", "Anti-Gravity Harness", "Anti-Matter Bomb", "Anti-Matter Drive", "Anti-Matter Torpedoes", "Anti-Missile Rockets", "Armor Barracks", "Artemis System Net", "Artifical Planet", "Assault Shuttles", "Astro University", "Atmospheric Renewer", "Augmented Engines", "Autolab", "Automated Factories", "Automated Repair Unit", "Battleoids", "Battle Pods", "Battle Scanner", "Battlestation", "Bio-Terminator", "Biomorphic Fungi", "Black Hole Generator", "Bomber Bays", "Capitol", "Class I Shield", "Class III Shield", "Class V Shield", "Class VII Shield", "Class X Shield", "Cloaking Device", "Cloning Center", "Colony Base", "Colony Ship", "Confederation", "Cyber-Security Link", "Cybertronic Computer", "Damper Field", "Dauntless Guidance System", "Death Ray", "Death Spores", "Deep Core Mining", "Core Waste Dumps", "Deuterium Fuel Cells", "Dimensional Portal", "Displacement Device", "Disrupter Cannon", "Doom Star Construction", "Reinforced Hull", "ECM Jammer", "Electronic Computer", "Emissions Guidance System", "Energy Absorber", "Biospheres", "Evolutionary Mutation", "Extended Fuel Tanks", "Fast Missile Racks", "Federation", "Fighter Bays", "Fighter Garrison", "Food Replicators", "Freighters", "Fusion Beam", "Fusion Bomb", "Fusion Drive", "Fusion Rifle", "Gaia Transformation", "Galactic Currency Exchange", "Galactic Cybernet", "Galactic Unification", "Gauss Auto-Cannon", "Graviton Beam", "Gyro Destabilizer", "Hard Shields", "Heavy Armor", "Heavy Fighter Bays", "Heightened Intelligence", "High Energy Focus", "Holo Simulator", "Hydroponic Farms", "Hyper Drive", "MegaFluxers", "Hyper-X Capacitors", "Hyperspace Communications", "Imperium", "Inertial Nullifier", "Inertial Stabilizer", "Interphased Drive", "Ion Drive", "Ion Pulse Cannon", "Iridium Fuel Cells", "Jump Gate", "Laser Cannon", "Laser Rifle", "Lightning Field", "Marine Barracks", "Mass Driver", "Mauler Device", "Merculite Missile", "Microbiotics", "Microlite Construction", "Outpost Ship", "Moleculartronic Computer", "Multi-Wave Ecm Jammer", "Multi-Phased Shields", "Nano Disassemblers", "Neural Scanner", "Neutron Blaster", "Neutron Scanner", "Neutronium Armor", "Neutronium Bomb", "Nuclear Bomb", "Nuclear Drive", "Nuclear Missile", "Optronic Computer", "Particle Beam", "Personal Shield", "Phase Shifter", "Phasing Cloak", "Phasor", "Phasor Rifle", "Planetary Barrier Shield", "Planetary Flux Shield", "Planetary Gravity Generator", "Planetary Missile Base", "Ground Batteries", "Planetary Radiation Shield", "Planetary Stock Exchange", "Planetary Supercomputer", "Plasma Cannon", "Plasma Rifle", "Plasma Torpedoes", "Plasma Web", "Pleasure Dome", "Pollution Processor", "Positronic Computer", "Powered Armor", "Pulse Rifle", "Proton Torpedoes", "Psionics", "Pulsar", "Pulson Missile", "Quantum Detonator", "Rangemaster Unit", "Recyclotron", "Reflection Field", "Robotic Factory", "Research Laboratory", "Robo-Miners", "Space Scanner", "Scout Lab", "Security Stations", "Sensors", "Shield Capacitors", "Soil Enrichment", "Space Academy", "Spaceport", "Spatial Compressor", "Spy Network", "Standard Fuel Cells", "Star Base", "Star Fortress", "Star Gate", "Stasis Field", "Stealth Field", "Stealth Suit", "Stellar Converter", "Structural Analyzer", "Sub-Space Communications", "Sub-Space Teleporter", "Subterranean Farms", "Survival Pods", "Tachyon Communications", "Tachyon Scanner", "Telepathic Training", "Terraforming", "Thorium Fuel Cells", "Time Warp Facilitator", "Titan Construction", "Titanium Armor", "Tractor Beam", "Transport", "Transporters", "Tritanium Armor", "Troop Pods", "Universal Antidote", "Uridium Fuel Cells", "Virtual Reality Network", "Warp Dissipater", "Warp Interdictor", "Weather Control System", "Wide Area Jammer", "Xeno Psychology", "Xentronium Armor", "Zeon Missile", "Zortrium Armor"]
    }



RESEARCH = {
    'construction':     {
        'index':            0,
        'start_area':       4,
    },
    'power':            {
        'index':            1,
        'start_area':       55,
    },
    'chemistry':        {
        'index':            2,
        'start_area':       22,
    },
    'sociology':        {
        'index':            3,
        'start_area':       10,
    },
    'computers':        {
        'index':            4,
        'start_area':       28,
    },
    'biology':          {
        'index':            5,
        'start_area':       18,
    },
    'physics':          {
        'index':            6,
        'start_area':       57,
    },
    'force_fields':     {
        'index':            7,
        'start_area':       7,
    },
}

RESEARCH_AREAS = {
    1:	{
	'name':		"Advanced Biology",
	'cost':		400,
	'next':		34,
    },
    2:	{
	'name':		"Advanced Chemistry",
	'cost':		650,
	'next':		47,
    },
    3:	{
	'name':		"Advanced Construction",
	'cost':		150,
	'next':		21,
    },
    4:	{
	'name':		"Advanced Engineering",
	'cost':		80,
	'next':		3,
    },
    5:	{
	'name':		"Advanced Fusion",
	'cost':		250,
	'next':		41,
    },
    6:	{
	'name':		"Advanced Governments",
	'cost':		4500,
	'next':		32,
    },
    7:	{
	'name':		"Advanced Magnetism",
	'cost':		250,
	'next':		36,
    },
    8:	{
	'name':		"Advanced Manufacturing",
	'cost':		1500,
	'next':		11,
    },
    9:	{
	'name':		"Advanced Metallurgy",
	'cost':		250,
	'next':		2,
    },
    10:	{
	'name':		"Military Tactics",
	'cost':		150,
	'next':		73,
    },
    11:	{
	'name':		"Advanced Robotics",
	'cost':		2000,
	'next':		67,
    },
    12:	{
	'name':		"Teaching Methods",
	'cost':		2000,
	'next':		6,
    },
    13:	{
	'name':		"Anti-Matter Fission",
	'cost':		2000,
	'next':		46,
    },
    14:	{
	'name':		"Artifical Consciousness",
	'cost':		1500,
	'next':		25,
    },
    15:	{
	'name':		"Artifical Intelligence",
	'cost':		400,
	'next':		60,
    },
    16:	{
	'name':		"Artifical Gravity",
	'cost':		1150,
	'next':		65,
    },
    17:	{
	'name':		"Artifical Life",
	'cost':		4500,
	'next':		70,
    },
    18:	{
	'name':		"Astro Biology",
	'cost':		80,
	'next':		1,
    },
    19:	{
	'name':		"Astro Construction",
	'cost':		1150,
	'next':		8,
    },
    20:	{
	'name':		"Astro Engineering",
	'cost':		400,
	'next':		62,
    },
    21:	{
	'name':		"Capsule Construction",
	'cost':		250,
	'next':		20,
    },
    22:	{
	'name':		"Chemistry",
	'cost':		50,
	'next':		9,
    },
    23:	{
	'name':		"Cold Fusion",
	'cost':		80,
	'next':		5,
    },
    24:	{
	'name':		"Cybertechnics",
	'cost':		3500,
	'next':		33,
    },
    25:	{
	'name':		"Cybertronics",
	'cost':		2750,
	'next':		24,
    },
    26:	{
	'name':		"Distortion Fields",
	'cost':		3500,
	'next':		61,
    },
    27:	{
	'name':		"Electromagnetic Refraction",
	'cost':		1500,
	'next':		72,
    },
    28:	{
	'name':		"Electronics",
	'cost':		50,
	'next':		56,
    },
    30:	{
	'name':		"Evolutionary Genetics",
	'cost':		2750,
	'next':		17,
    },
    31:	{
	'name':		"Fusion Physics",
	'cost':		150,
	'next':		66,
    },
    32:	{
	'name':		"Galactic Economics",
	'cost':		6000,
	'next':		82,
    },
    33:	{
	'name':		"Galactic Networking",
	'cost':		4500,
	'next':		49,
    },
    34:	{
	'name':		"Genetic Engineering",
	'cost':		900,
	'next':		35,
    },
    35:	{
	'name':		"Genetic Mutations",
	'cost':		1150,
	'next':		44,
    },
    36:	{
	'name':		"Gravitic Fields",
	'cost':		650,
	'next':		45,
    },
    37:	{
	'name':		"High Energy Distribution",
	'cost':		3500,
	'next':		38,
    },
    38:	{
	'name':		"Hyper Dimensioanl Fission",
	'cost':		4500,
	'next':		40,
    },
    39:	{
	'name':		"Hyper Dimensional Physics",
	'cost':		6000,
	'next':		69,
    },
    40:	{
	'name':		"Interphased Fission",
	'cost':		10000,
	'next':		76,
    },
    41:	{
	'name':		"Ion Fission",
	'cost':		900,
	'next':		13,
    },
    42:	{
	'name':		"Superscalar Construction",
	'cost':		6000,
	'next':		58,
    },
    43:	{
	'name':		"Macro Economics",
	'cost':		1150,
	'next':		12,
    },
    44:	{
	'name':		"Macro Genetics",
	'cost':		1500,
	'next':		30,
    },
    45:	{
	'name':		"Magneto Gravitics",
	'cost':		900,
	'next':		27,
    },
    46:	{
	'name':		"Matter Energy Conversion",
	'cost':		2750,
	'next':		37,
    },
    47:	{
	'name':		"Molecular Compression",
	'cost':		1150,
	'next':		53,
    },
    48:	{
	'name':		"Molecular Control",
	'cost':		10000,
	'next':		80,
    },
    49:	{
	'name':		"Moleculartronics",
	'cost':		6000,
	'next':		81,
    },
    50:	{
	'name':		"Molecular Manipulation",
	'cost':		4500,
	'next':		48,
    },
    51:	{
	'name':		"Multi-Dimensional Physics",
	'cost':		4500,
	'next':		39,
    },
    52:	{
	'name':		"Multi-Phased Physics",
	'cost':		2000,
	'next':		59,
    },
    53:	{
	'name':		"Nano Technology",
	'cost':		2000,
	'next':		50,
    },
    54:	{
	'name':		"Neutrino Physics",
	'cost':		900,
	'next':		16,
    },
    55:	{
	'name':		"Nuclear Fission",
	'cost':		50,
	'next':		23,
    },
    56:	{
	'name':		"Optronic",
	'cost':		150,
	'next':		15,
    },
    57:	{
	'name':		"Physics",
	'cost':		50,
	'next':		31,
    },
    58:	{
	'name':		"Planetoid Construction",
	'cost':		7500,
	'next':		78,
    },
    59:	{
	'name':		"Plasma Physics",
	'cost':		3500,
	'next':		51,
    },
    60:	{
	'name':		"Positronics",
	'cost':		900,
	'next':		14,
    },
    61:	{
	'name':		"Quantum Fileds",
	'cost':		4500,
	'next':		71,
    },
    62:	{
	'name':		"Robotics",
	'cost':		650,
	'next':		63,
    },
    63:	{
	'name':		"Servo Mechanics",
	'cost':		900,
	'next':		19,
    },
    64:	{
	'name':		"Subspace Fields",
	'cost':		2750,
	'next':		26,
    },
    65:	{
	'name':		"Subspace Physics",
	'cost':		1500,
	'next':		52,
    },
    66:	{
	'name':		"Tachyon Physics",
	'cost':		250,
	'next':		54,
    },
    67:	{
	'name':		"Tectonic Engineering",
	'cost':		3500,
	'next':		42,
    },
    68:	{
	'name':		"Temporal Fields",
	'cost':		15000,
	'next':		79,
    },
    69:	{
	'name':		"Temporal Physics",
	'cost':		15000,
	'next':		77,
    },
    70:	{
	'name':		"Trans Genetics",
	'cost':		7500,
	'next':		75,
    },
    71:	{
	'name':		"Transwarp Fields",
	'cost':		7500,
	'next':		68,
    },
    72:	{
	'name':		"Warp Fields",
	'cost':		2000,
	'next':		64,
    },
    73:	{
	'name':		"Xeno Relations",
	'cost':		650,
	'next':		43,
    },
    75:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    76:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    77:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    78:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    79:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    80:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    81:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    82:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
}



import pygame

from lbx import load_surface
import get_input

#import formulas

from _game_constants import *

import colony_build_screen


def print_summary_item(data, key, title):
    if data.has_key(key) and data[key]:
	print "+     % 6s ... % 25s    +" % (str(data[key]), title)

def draw_planet_background(GUI, DISPLAY, IMAGES, PALETTES, planet_terrain, bg_pic_id):
    bg_pics = [0, 3, 6, 9, 12, 15, 18, 23, 24, 27]
    pic = bg_pics[planet_terrain] + bg_pic_id

    if IMAGES['PLANET_BACKGROUND'][pic] == None:
	print "Lazy load of PLANET_BACKGROUND #%i" % pic
	GUI.load_planet_background(pic)
#	IMAGES['PLANET_BACKGROUND'][pic] = load_surface(LBX['PLANETS.LBX'].read_picture(pic), 0, PALETTES['PLANETS.LBX'])

    DISPLAY.blit(IMAGES['COLONY_SCREEN']['background'], (0, 0))
    DISPLAY.blit(IMAGES['PLANET_BACKGROUND'][pic], (0, 0))

#
#       DRAW
#
def draw(GAME, star, planet, colony):
    GUI		= GAME['GUI']
    DISPLAY     = GAME['DISPLAY']
    IMAGES      = GAME['IMAGES']
    PALETTES	= GAME['PALETTES']
    FONTS	= GAME['FONTS']

    DICTIONARY	= GAME['DICTIONARY']
    
    PLAYERS	= GAME['DATA']['players']

    buttons = []

    print("=== colony_screen::draw() ... star:")
    print("star:")
    print(str(star))
    print("/star")

#    owner = colony.owner
#    colony_name	= "%s %i" % (star['name'], planet['position'])

    draw_planet_background(GUI, DISPLAY, IMAGES, PALETTES, planet.get_terrain(), planet.get_picture())

#    pic = pics[planet['terrain']] + planet['picture']

    DISPLAY.blit(IMAGES['COLONY_SCREEN']['panel'], (0, 0))

#    DISPLAY.blit(IMAGES['COLONY_SCREEN']['planet_arrow'], (6,  31))
#    DISPLAY.blit(IMAGES['COLONY_SCREEN']['planet_arrow'], (6,  55))
#    DISPLAY.blit(IMAGES['COLONY_SCREEN']['planet_arrow'], (6,  79))
#    DISPLAY.blit(IMAGES['COLONY_SCREEN']['planet_arrow'], (6, 103))
#    DISPLAY.blit(IMAGES['COLONY_SCREEN']['planet_arrow'], (6, 127))
    
    for i in range(5):
	object_id = star.get_objects()[i]
	if object_id != 0xFFFF:
	    object = GAME['DATA']['planets'][object_id]
	    print "type: %i" % object.get_type()

	    if object.is_asteroid_belt():
		x = 6
		y = 22 + (24 * i)
		DISPLAY.blit(IMAGES['COLONY_SCREEN']['asteroids_scheme'], (x, y))
		DISPLAY.blit(FONTS['font_10'].render("Asteroids", 1, (0x6C, 0x68, 0x8C)), (x + 30, y + 6))

	    if object.is_gas_giant():
		x = 11
		y = 27 + (24 * i)
		DISPLAY.blit(IMAGES['COLONY_SCREEN']['gasgiant_scheme'], (x, y))
		DISPLAY.blit(FONTS['font_10'].render("Gas Giant -", 1, (0x6C, 0x68, 0x8C)), (x + 24, y + 2))
		DISPLAY.blit(FONTS['font_10'].render("uninhabitable", 1, (0x6C, 0x68, 0x8C)), (x + 24, y + 12))

	    elif object.is_planet():
		terrain = object.get_terrain()
		size = object.get_size()
		x = 10 + [6, 4, 3, 1, 0][size]
		y = 26 + (24 * i) + [6, 4, 2, 1, 0][size]
	        DISPLAY.blit(IMAGES['COLONY_SCREEN']['planet_schemes'][terrain][size], (x, y))
	DISPLAY.blit(IMAGES['COLONY_SCREEN']['planet_arrow'], (6,  31 + (24 * i)))
    
    title = "%s of %s" % (DICTIONARY['COLONY_ASSIGNMENT'][colony.assignment], colony.get_name())
    (tw, th) = FONTS['font_14_bold'].size(title)
    
#    DISPLAY.blit(FONTS['font_16_bold'].render(title, 1, (0x20, 0x28, 0x4c)), (320 - (tw / 2), 1))
#    DISPLAY.blit(FONTS['font_16_bold'].render(title, 1, (0x50, 0x6c, 0x90)), (319 - (tw / 2), 0))
    DISPLAY.blit(FONTS['font_14_bold'].render(title, 1, (0x68, 0x68, 0x88)), (320 - (tw / 2), 1))

#    total_population = (1000 * colony['population']) + colony['pop_raised']
#    print
#    print "	Colony:		%s" % colony.name
#    print "	Population:	%i (+%i)" % (total_population, colony['pop_grow'])
#    print "	Industry:	%i" % colony.industry()
#    print "	Research:	%i" % colony.research()
#    print "	Food (result):	%i (%i)" % (colony.food(), colony.food() - colony.population)
#    print "	Colonists:"

    DISPLAY.blit(IMAGES['GOVERNMENT_ICONS'][PLAYERS[colony.get_owner()].get_racepick_item('goverment')], (310, 32))

    def repeat_draw(target_surface, x, y, source_surface, number, icon_width, break_count, area_width):
#	print pos
#	return
	if number < break_count:
	    xx = icon_width
	else:
	    xx = int(area_width / number)
	for i in range(int(number)):
	    target_surface.blit(source_surface, (x, y))
	    x += xx
	return x
    # end func repeat_draw
    repeat_draw(DISPLAY, 340, 35, IMAGES['MORALE_ICONS']['good'], colony.morale() // 10, 30, 7, 155)

    x = 10 + repeat_draw(DISPLAY, 128, 64, IMAGES['COLONY2.LBX']['10food'], colony.get_food() // 10, 20, 6, 98)
    repeat_draw(DISPLAY, x, 64, IMAGES['COLONY2.LBX']['1food'], colony.get_food() % 10, 20, 6, 98)

    # industry icons
    number = (colony.get_industry() // 10) + (colony.get_industry() % 10)
    xx = min(int(round(160 / max(1, number))), 20)
    print "### colony_screen::draw ... industry icons ... number = %i, xx = %i" % (number, xx)
    x = repeat_draw(DISPLAY, 128, 94, IMAGES['COLONY2.LBX']['10production'], colony.get_industry() // 10, xx, 99, 162)
    repeat_draw(DISPLAY, x, 94, IMAGES['COLONY2.LBX']['1production'], colony.get_industry() % 10, xx, 99, 162)

    # research icons
    
    number = (colony.get_research() // 10) + (colony.get_research() % 10)
    xx = min(int(round(160 / max(1, number))), 20)
    print "### colony_screen::draw ... research icons ... number = %i, xx = %i" % (number, xx)
    x = repeat_draw(DISPLAY, 128, 124, IMAGES['COLONY2.LBX']['10research'], colony.get_research() // 10, xx, 99, 162)
    repeat_draw(DISPLAY, x, 124, IMAGES['COLONY2.LBX']['1research'], colony.get_research() % 10, xx, 99, 162)

#    print colony['colonists']

    for t in (FARMER, WORKER, SCIENTIST):
	c = len(colony.colonists[t])
#	c = 90
#	for i in range(c):
	if c < 7:
	    xx = 30
	else:
	    xx = 190 / c

	if t == 0x02:
	    icon = 1
	    y = 62
	elif t == 0x82:
	    icon = 3
	    y = 92
	elif t == 0x03:
	    icon = 5
	    y = 122

	for i in range(c):
	    Colonist = colony.colonists[t][i]
#	    colonist = colony['colonists'][t][0]
	    race = Colonist['race']
	    picture = PLAYERS[race].get_picture()
	    x = 310 + xx * i
#	    print "		%.2x %.2x %.2x %.2x" % (Colonist['a'], Colonist['b'], Colonist['c'], Colonist['d'])
	    DISPLAY.blit(IMAGES['RACEICON.LBX'][picture][icon], (x, y))
	    if i == (c - 1):
		xx = 28	# enlarge the Rect of last icon (no other icon is drawn over it...)
	    buttons.append({'action': "pick-colonist:%.2x:%i" % (t, (c - i)),    'rect': pygame.Rect((x, y), (xx, 28))})

    x = 0
    for i in range(colony.marines):
	DISPLAY.blit(IMAGES['RACEICON.LBX'][picture][0x07], (x, 450))
	x += 30

    # TODO: count in all races not just owner!
    total_population = (1000 * colony.total_population()) + sum(colony.pop_raised())
    pop_s = "Pop %i,%.3i k (+%i)" % ((total_population // 1000), (total_population % 1000), sum(colony.pop_grow()))

    (tw, th) = FONTS['font_12'].size(pop_s)

    DISPLAY.blit(FONTS['font_12'].render(pop_s, 1, (0x68, 0x68, 0x88)), (639 - tw, 1))
        
    pygame.display.flip()

    return buttons
# end func draw

##
#       RUN
##
def run(GAME, colony_id):
#    draw(GAME)

    DATA	= GAME['DATA']
#    RULES       = DATA['rules']
#    PLAYERS     = DATA['players']

    print
    print "@@@ colony_screen @@@"
#    print "     planet_id: %i" % planet_id

    colony      = DATA['colonies'][colony_id]

    planet_id	= colony.get_planet_id()
    planet      = DATA['planets'][planet_id]

    star_id	= planet.get_star()
    star	= DATA['stars'][star_id]

#    print "colony_screen::run() ... buildings = %s" % colony.buildings
    print "colony_screen::run() ... colony.build_queue[] = %s" % colony.build_queue()[0]
    print "colony_screen::run() ... colony.build_queue[] = %s" % colony.build_queue()[1]
    print "colony_screen::run() ... colony.build_queue[] = %s" % colony.build_queue()[2]
    print "colony_screen::run() ... colony.build_queue[] = %s" % colony.build_queue()[3]
    print "colony_screen::run() ... colony.build_queue[] = %s" % colony.build_queue()[4]
    print "colony_screen::run() ... colony.build_queue[] = %s" % colony.build_queue()[5]
    print "colony_screen::run() ... colony.build_queue[] = %s" % colony.build_queue()[6]

#    colony.recount(RULES, PLAYERS)

#    draw(GAME, planet['terrain'], planet['picture'], "%s %i" % (star['name'], planet['position']))
    

    triggers = [
        {'action': "ESCAPE",    		'rect': pygame.Rect((556, 459), ( 72, 20))},
        {'action': "leaders",   		'rect': pygame.Rect((556, 427), ( 72, 20))},
        {'action': "change_build",    		'rect': pygame.Rect((519, 123), ( 61, 22))},
        {'action': "buy",       		'rect': pygame.Rect((590, 123), ( 37, 22))},
	{'action': "morale_summary",		'rect': pygame.Rect((309,  31), (202, 29))},
	{'action': "bc_summary",		'rect': pygame.Rect((130,  31), (179, 29))},
	{'action': "food_summary",		'rect': pygame.Rect((130,  61), (179, 26))},
	{'action': "industry_summary",		'rect': pygame.Rect((130,  91), (179, 26))},
	{'action': "research_summary",		'rect': pygame.Rect((130, 121), (179, 26))}
    ]

    triggers2 = draw(GAME, star, planet, colony)

    while True:
        event = get_input.get_input(triggers + triggers2)
        action = event['action']

        if (action == "ESCAPE"):
            return

	elif (action == "morale_summary"):
	    colony.print_morale_summary()

	elif (action == "bc_summary"):
	    colony.print_bc_summary()

	elif (action == "food_summary"):
	    colony.print_food_summary()

	elif (action == "industry_summary"):
	    colony.print_industry_summary()

	elif (action == "research_summary"):
	    colony.print_research_summary()

	elif (action == "change_build"):
	    colony_build_screen.run(GAME, colony_id)
            triggers2 = draw(GAME, star, planet, colony)

        else:
            print "UNKNONW ACTION: " + action

# end func run




__author__="peterman"
__date__ ="$Jan 7, 2009 8:44:49 PM$"

import pygame

import get_input

#from _technology import *


def list_techs_by_area(tech_table, research_area):
    techs = {}
    for tech_id in tech_table:
        if tech_table[tech_id]['area'] == research_area:
            techs[tech_id] = tech_table[tech_id]
    return techs
# end func list_techs_by_area

def debug_known_techs(known_techs, tech_table):
    print("")
    print("=== Known Technologies: ===")
    for tech_id in known_techs:
        print("technology = %i ... %s" % (tech_id, tech_table[tech_id]))
    print("=== /Known Technologies ===")
    print("")


##
#       DRAW
##
def draw(GAME, hover):
    DATA        = GAME['DATA']
    RULES       = DATA['rules']
    ME          = DATA['me']

    DISPLAY     = GAME['DISPLAY']
    FONTS       = GAME['FONTS']
    IMAGES      = GAME['IMAGES']
    DISPLAY.blit(IMAGES['RESEARCH_DIALOG']['panel'], (80, 0))

#    print("research_screen::draw() ... hover = %s" % hover)

    dynamic_triggers = []

    research_areas = ME.list_research_areas()

#    print research_areas
    
    for research in research_areas:
        research_index = RULES['research'][research]['index']
#        print "research_index = %i" % research_index
        first_tech_id = research_areas[research][0]
        first_tech = RULES['tech_table'][first_tech_id]
        area_id = first_tech['area']

        if ME.get_research_area() == area_id:
            color = (0x64, 0xd0, 0x00)
        else:
            color = (0x04, 0x78, 0x00)

#        print RULES['research_areas'][area_id]

        x = 95 + (226 * (research_index % 2))
        y = 50 + (105 * (research_index // 2))

        label = FONTS['font_14_bold'].render(RULES['research_areas'][area_id]['name'], 1, color)

        DISPLAY.blit(label, (x, y))

#        techs = list_techs_by_area(RULES['tech_table'], my_area)
        i = 0
        y += 16
        x += 10
        for tech_id in research_areas[research]:
#            print techs[tech_id]

            if (hover is not None) and (hover['action'] == "set_research") and (hover['tech_id'] == tech_id):
#                tech_color = (0x00, 0xff, 0x00)
                tech_color = (0xAA, 0xFF, 0xAA)
            else:
                tech_color = color

            label = FONTS['font_13_bold'].render(RULES['tech_table'][tech_id]['name'], 1, tech_color)
            l_w = label.get_width()
            l_h = label.get_height()

            yy = i * 16
            DISPLAY.blit(label, (x, y + yy))
            dynamic_triggers.append({'action': "set_research", 'tech_id': tech_id, 'rect': pygame.Rect((x, y + yy), (l_w, l_h))})
            i += 1

    pygame.display.flip()

    return dynamic_triggers
# end func draw

##
#       RUN
##
def run(GAME):

    DATA        = GAME['DATA']
    RULES       = DATA['rules']

    debug_known_techs(DATA['me'].get_known_techs(), RULES['tech_table'])

    hover = None

    triggers = [
        {'action': "ESCAPE",        'rect': pygame.Rect((0, 0), (79, 480))},
        {'action': "ESCAPE",        'rect': pygame.Rect((552, 0), (87, 480))}
    ] + draw(GAME, hover)


    while True:
        event = get_input.get_input(triggers)
        action = event['action']

        if action == "ESCAPE":
            return

        elif action == "set_research":
            tech_id = event['tech_id']
            print(">>> set_research ... tech_id = %i = %s" % (tech_id, RULES['tech_table'][tech_id]['name']))
	    GAME['DATA']['me'].print_research_debug()
	    GAME['DATA'] = GAME['client'].set_research(tech_id)
	    GAME['DATA']['me'].print_research_debug()
            return


        elif action == "help":
            print(">>> help ... help = %s" % (event['help']))

        elif action == "hover":
#            print(">>> HOVER ... hover = %s" % (event['hover']))
            if hover != event['hover']:
                hover = event['hover']
                triggers = [
                    {'action': "ESCAPE",        'rect': pygame.Rect((0, 0), (79, 480))},
                    {'action': "ESCAPE",        'rect': pygame.Rect((552, 0), (87, 480))}
                ] + draw(GAME, hover)

        else:
            print "UNKNONW ACTION: " + action
    

